@@ -68,11 +68,11 @@ typedef struct {
     unsigned int axisMask;	/* Indicates which axes to map element's
 				 * coordinates onto */
     Vector x, y;		/* Contains array of numeric values */
-    DataStatus *statusArr;	/* Array of flags (malloc-ed) which
+    int *activeIndexArr;	/* Array of indices (malloc-ed) which
 				 * indicate which data points are active
-				 * (drawn with "active" colors. */
-    int arraySize;		/* Number of active data points. Special
-				 * case: if numActivePoints==0 and the active
+				 * (drawn with "active" colors). */
+    int numActiveIndices;	/* Number of active data points. Special
+				 * case: if numActiveIndices==0 and the active
 				 * bit is set in "flags", then all data
 				 * points are drawn active. */
 
@@ -88,9 +88,6 @@ typedef struct {
     ElemDrawSymbolsProc *drawSymbolsProc;
     ElemPrintSymbolsProc *printSymbolsProc;
 
-    int numDataPoints;		/* Number of data points (x-y cooridinates)
-				 * in the element.  This is the minimum
-				 * size of the x and y vectors. */
     /*
      * Bar specific attributes
      */
@@ -105,14 +102,15 @@ typedef struct {
     int borderWidth;		/* 3D border width of bar */
     int relief;			/* Relief of the bar */
 
-
-    XRectangle defSpace[DEF_NUM_BARS];
+    int defIndexSpace[DEF_NUM_BARS];
+    int *indexArr;
+    XRectangle defSegSpace[DEF_NUM_BARS];
     XRectangle *segArr;		/* Array of rectangles comprising the bar
 				 * segments of the element. By default, this
 				 * is set to *defSpace*.  If more segments are
 				 * needed, the array will point to malloc-ed
 				 * memory */
-    int numSegs;		/* Number of bars to be drawn */
+    int numVisibleSegments;	/*  */
     int padX;			/* Spacing on either side of bar */
     double barWidth;
 
@@ -192,11 +190,18 @@ static Tk_ConfigSpec configSpecs[] =
     {TK_CONFIG_END, NULL, NULL, NULL, NULL, 0, 0}
 };
 
-static INLINE int
-Abs(x)
-    register int x;
+INLINE static double
+Fabs(x)
+    register double x;
 {
-    return ((x < 0) ? -x : x);
+    return ((x < 0.0) ? -x : x);
+}
+
+INLINE static int
+Round(x)
+    register double x;
+{
+    return (int) (x + ((x < 0.0) ? -0.5 : 0.5));
 }
 
 /* ----------------------------------------------------------------------
@@ -392,48 +397,49 @@ ComputeLimits(graphPtr, elemPtr, axisPtr, minPtr, maxPtr)
     double *minPtr, *maxPtr;
 {
     Bar *barPtr = (Bar *)elemPtr;
-    Vector *vecPtr;
+    Vector *vPtr;
     double min, max;
 
-    min = bltPosInfinity, max = bltNegInfinity;
-    if ((barPtr->x.length < 1) || (barPtr->y.length < 1)) {
+    if (ELEMENT_LENGTH(barPtr) < 1) {
 	return 0;		/* No data points */
     }
     if (!(elemPtr->axisMask & AXIS_MASK(axisPtr))) {
 	return 0;		/* Bar is not mapped to this axis */
     }
+    min = bltPosInfinity, max = bltNegInfinity;
     if (AXIS_MASK(axisPtr) & AXIS_MASK_X) {
-	double middle;
+	double middle, barWidth;
 
-	vecPtr = &(barPtr->x);
-	/* This doesn't help much if the width of the bars on the
-	 * ends have been configured differently. But I don't know
-	 * that information until I lay out the bar elements */
-	middle = graphPtr->barWidth * 0.5;
-	min = vecPtr->min - middle;
-	max = vecPtr->max + middle;
+	barWidth = graphPtr->barWidth;
+	if (barPtr->barWidth > 0.0) {
+	    barWidth = barPtr->barWidth;
+	}
+	vPtr = &(barPtr->x);
+	middle = barWidth * 0.5;
+	min = vPtr->vector.min - middle;
+	max = vPtr->vector.max + middle;
 
 	/* Warning: You get what you deserve if the x-axis is logScale */
 	if (axisPtr->logScale) {
-	    min = Blt_FindVectorMinimum(vecPtr, DBL_MIN) + middle;
+	    min = Blt_FindVectorMinimum(vPtr, DBL_MIN) + middle;
 	}
     } else {
-	vecPtr = &(barPtr->y);
-	min = barPtr->y.min;
+	vPtr = &(barPtr->y);
+	min = barPtr->y.vector.min;
 	if ((min <= 0.0) && (axisPtr->logScale)) {
-	    min = Blt_FindVectorMinimum(vecPtr, DBL_MIN);
+	    min = Blt_FindVectorMinimum(vPtr, DBL_MIN);
 	} else {
 	    if (min > 0.0) {
 		min = 0.0;
 	    }
 	}
-	max = barPtr->y.max;
+	max = barPtr->y.vector.max;
 	if (max < 0.0) {
 	    max = 0.0;
 	}
     }
     *minPtr = min, *maxPtr = max;
-    return (vecPtr->length);
+    return (vPtr->vector.numValues);
 }
 
 /*
@@ -468,18 +474,16 @@ ClosestBar(graphPtr, elemPtr, infoPtr)
 
     prevMin = infoPtr->dist;
     rectPtr = barPtr->segArr;
-    for (i = 0; i < barPtr->numDataPoints; i++) {
-	if (IsClipped(barPtr->statusArr[i])) {
-	    continue;		/* Ignore bar segments that aren't visible */
-	}
+    for (i = 0; i < barPtr->numVisibleSegments; i++) {
 	if ((infoPtr->x >= rectPtr->x) &&
 	    (infoPtr->x <= (rectPtr->x + rectPtr->width)) &&
 	    (infoPtr->y >= rectPtr->y) &&
 	    (infoPtr->y <= (rectPtr->y + rectPtr->height))) {
 
-	    /* The point is interior to this bar segment. */
-
-	    infoPtr->index = i;
+	    /*
+	     * The point is interior to this bar segment.
+	     */
+	    infoPtr->index = barPtr->indexArr[i];
 	    infoPtr->dist = 0.0;
 	    break;
 	}
@@ -493,7 +497,7 @@ ClosestBar(graphPtr, elemPtr, infoPtr)
 	    dist = Blt_GetDistanceToSegment(pointPtr, infoPtr->x, infoPtr->y,
 		&dummy, &dummy);
 	    if (dist < infoPtr->dist) {
-		infoPtr->index = i;
+		infoPtr->index = barPtr->indexArr[i];
 		infoPtr->dist = dist;
 	    }
 	    pointPtr++;
@@ -503,8 +507,8 @@ ClosestBar(graphPtr, elemPtr, infoPtr)
     if (infoPtr->dist < prevMin) {
 	infoPtr->elemId = elemPtr->nameId;
 	/* Provide the segment value, not the stacked sum */
-	infoPtr->closest.x = barPtr->x.data[infoPtr->index];
-	infoPtr->closest.y = barPtr->y.data[infoPtr->index];
+	infoPtr->closest.x = barPtr->x.vector.valueArr[infoPtr->index];
+	infoPtr->closest.y = barPtr->y.vector.valueArr[infoPtr->index];
     }
 }
 
@@ -536,61 +540,71 @@ ComputeBarCoordinates(graphPtr, elemPtr)
     Graph *graphPtr;
     Element *elemPtr;
 {
-    AxisPair axes;
+    AxisPair axisPair;
     Bar *barPtr = (Bar *)elemPtr;
-    int numDataPoints;
     double x1, y1, x2, y2;	/* Two opposite corners of the rectangle
 				 * in graph coordinates. */
-    XPoint corner1, corner2;
+    Coordinate corner1, corner2;
     double barWidth;
     FreqKey key;
-    register XRectangle *rectPtr;
-    register int i, count;
+    int numDataPoints, numVisible;
+    register XRectangle *rectPtr, *rectArr, *segPtr;
+    register int *indexPtr;
+    register int i;
+    int invertBar;
+    int height;
+    double *xArr, *yArr;
 
-    if ((barPtr->x.length < 1) || (barPtr->y.length < 1)) {
-	return;			/* No data points */
-    }
-    if (barPtr->segArr != barPtr->defSpace) {
+    /* Release any storage associated with the display of the bar */
+    if (barPtr->segArr != barPtr->defSegSpace) {
 	free((char *)barPtr->segArr);
     }
-    barPtr->numDataPoints = barPtr->numSegs = 0;
-    barPtr->segArr = barPtr->defSpace;
-    numDataPoints = BLT_MIN(barPtr->x.length, barPtr->y.length);
-    if (numDataPoints > DEF_NUM_BARS) {
-	XRectangle *segArr;
+    barPtr->segArr = barPtr->defSegSpace;
+    barPtr->numVisibleSegments = 0;
+    if (barPtr->indexArr != barPtr->defIndexSpace) {
+	free((char *)barPtr->indexArr);
+    }
+    barPtr->indexArr = barPtr->defIndexSpace;
 
-	segArr = (XRectangle *)malloc(numDataPoints * sizeof(XRectangle));
-	if (segArr == NULL) {
-	    return;		/* Can't allocated bar array */
-	}
-	barPtr->segArr = segArr;
+    numDataPoints = ELEMENT_LENGTH(barPtr);
+    if (numDataPoints < 1) {
+	return;			/* No data points */
     }
-    if (numDataPoints != elemPtr->arraySize) {
-	Blt_ResizeStatusArray(elemPtr, numDataPoints);
-    }
-    Blt_GetAxisMapping(graphPtr, barPtr->axisMask, &axes);
     barWidth = graphPtr->barWidth;
     if (barPtr->barWidth > 0.0) {
 	barWidth = barPtr->barWidth;
     }
+    Blt_GetAxisMapping(graphPtr, barPtr->axisMask, &axisPair);
+
+    /*
+     * Create an array of rectangles representing the screen coordinates
+     * of all the segments in the bar.  Note that not all segments may be
+     * visible (i.e. some may be clipped entirely).  Segments not visible
+     * will have a width and height of zero.
+     */
+    rectArr = (XRectangle *)calloc(numDataPoints, sizeof(XRectangle));
+    if (rectArr == NULL) {
+	return;			/* Can't allocate bar array */
+    }
+    xArr = barPtr->x.vector.valueArr;
+    yArr = barPtr->y.vector.valueArr;
     key.dummy = 0;
-    rectPtr = barPtr->segArr;
-    count = 0;
+    numVisible = 0;
+    rectPtr = rectArr;
     for (i = 0; i < numDataPoints; i++) {
-	SetClipped(barPtr->statusArr[i]);	/* Turn off visibility */
-	if (((barPtr->x.data[i] - barWidth) > axes.x->max) ||
-	    ((barPtr->x.data[i] + barWidth) < axes.x->min)) {
+	if (((xArr[i] - barWidth) > axisPair.x->max) ||
+	    ((xArr[i] + barWidth) < axisPair.x->min)) {
 	    continue;		/* Abscissa is out of range of the x-axis */
 	}
-	x1 = barPtr->x.data[i] - (barWidth * 0.5);
+	x1 = xArr[i] - (barWidth * 0.5);
 	x2 = x1 + barWidth;
-	y1 = barPtr->y.data[i];
+	y1 = yArr[i];
 	y2 = 0.0;
 
 	if ((graphPtr->numStacks > 0) && (graphPtr->mode != MODE_NORMAL)) {
 	    Tcl_HashEntry *hPtr;
 
-	    key.value = barPtr->x.data[i];
+	    key.value = xArr[i];
 	    key.mask = barPtr->axisMask;
 	    hPtr = Tcl_FindHashEntry(&(graphPtr->freqTable), (char *)&key);
 	    if (hPtr != NULL) {
@@ -611,39 +625,88 @@ ComputeBarCoordinates(graphPtr, elemPtr)
 		}
 	    }
 	}
+	invertBar = FALSE;
 	if (y1 < y2) {
 	    double temp;
 
 	    /* Handle negative bar values by swapping ordinates */
 	    temp = y1, y1 = y2, y2 = temp;
+	    invertBar = TRUE;
 	}
+#ifdef notdef
+	fprintf(stderr, "x1=%g,y1=%g x2=%g,y2=%g\n", x1, y1, x2, y2);
+#endif
 	/*
 	 * Get the two corners of the bar segment and compute the rectangle
 	 */
-	corner1 = Blt_TransformPt(graphPtr, x1, y1, &axes);
-	corner2 = Blt_TransformPt(graphPtr, x2, y2, &axes);
+	corner1 = Blt_TransformPt(graphPtr, x1, y1, &axisPair);
+	corner2 = Blt_TransformPt(graphPtr, x2, y2, &axisPair);
 
 	/* Clip the bars vertically at the size of the graph window */
-	if (corner1.y < 0) {
-	    corner1.y = 0;
-	} else if (corner1.y > graphPtr->height) {
-	    corner1.y = graphPtr->height;
+	if (corner1.y < 0.0) {
+	    corner1.y = 0.0;
+	} else if (corner1.y > (double)graphPtr->height) {
+	    corner1.y = (double)graphPtr->height;
 	}
-	if (corner2.y < 0) {
-	    corner2.y = 0;
-	} else if (corner2.y > graphPtr->height) {
-	    corner2.y = graphPtr->height;
+	if (corner2.y < 0.0) {
+	    corner2.y = 0.0;
+	} else if (corner2.y > (double)graphPtr->height) {
+	    corner2.y = (double)graphPtr->height;
 	}
-	rectPtr->x = BLT_MIN(corner1.x, corner2.x);
-	rectPtr->y = BLT_MIN(corner1.y, corner2.y);
-	rectPtr->width = Abs(corner1.x - corner2.x);
-	rectPtr->height = Abs(corner1.y - corner2.y);
-	SetVisible(barPtr->statusArr[i]);
+	height = (int)Round(Fabs(corner1.y - corner2.y));
+	if (invertBar) {
+	    rectPtr->y = (int)BLT_MIN(corner1.y, corner2.y);
+	} else {
+	    rectPtr->y = (int)(BLT_MAX(corner1.y, corner2.y)) - height;
+	}
+	rectPtr->x = (int)BLT_MIN(corner1.x, corner2.x);
+	rectPtr->width = (int)Round(Fabs(corner1.x - corner2.x));
+	if (rectPtr->width < 1) {
+	    rectPtr->width = 1;
+	}
+	rectPtr->height = height;
+	if (rectPtr->height < 1) {
+	    rectPtr->height = 1;
+	}
+#ifdef notdef
+	fprintf(stderr, "R%d: x=%d,y=%d,w=%d,h=%d\n", i, rectPtr->x, rectPtr->y,
+	    rectPtr->width, rectPtr->height);
+	fprintf(stderr, "c1 = (x=%g,y=%g) c2 = (x=%g,y=%g)\n", corner1.x,
+	    corner1.y, corner2.x, corner2.y);
+#endif
+	numVisible++;
 	rectPtr++;
-	count++;
     }
-    barPtr->numSegs = count;
-    barPtr->numDataPoints = numDataPoints;
+    barPtr->numVisibleSegments = numVisible;
+
+    if (numVisible > DEF_NUM_BARS) {
+	XRectangle *segArr;
+	int *indexArr;
+
+	segArr = (XRectangle *)malloc(numVisible * sizeof(XRectangle));
+	if (segArr == NULL) {
+	    Panic("can't allocate bar array");
+	}
+	barPtr->segArr = segArr;
+	indexArr = (int *)malloc(numVisible * sizeof(int));
+	if (indexArr == NULL) {
+	    Panic("can't allocate index array");
+	}
+	barPtr->indexArr = indexArr;
+    }
+    /* Collect the visible segments into a single array */
+    rectPtr = rectArr;
+    segPtr = barPtr->segArr;
+    indexPtr = barPtr->indexArr;
+    for (i = 0; i < numDataPoints; i++) {
+	if ((rectPtr->width > 0) && (rectPtr->height > 0)) {
+	    *segPtr = *rectPtr;
+	    *indexPtr = i;
+	    segPtr++, indexPtr++;
+	    rectPtr++;
+	}
+    }
+    free((char *)rectArr);
 }
 
 /*
@@ -711,17 +774,17 @@ DrawNormalBar(graphPtr, elemPtr)
     Element *elemPtr;
 {
     Bar *barPtr = (Bar *)elemPtr;
+    register XRectangle *rectPtr;
+    register int i;
 
     XFillRectangles(graphPtr->display, graphPtr->pixwin, barPtr->normalGC,
-	barPtr->segArr, barPtr->numSegs);
+	barPtr->segArr, barPtr->numVisibleSegments);
     if ((barPtr->borderWidth > 0) && (barPtr->relief != TK_RELIEF_FLAT)) {
-	register XRectangle *rectPtr;
-	register int i;
 	int twiceBW;
 
 	twiceBW = (2 * barPtr->borderWidth);
 	rectPtr = barPtr->segArr;
-	for (i = 0; i < barPtr->numSegs; i++) {
+	for (i = 0; i < barPtr->numVisibleSegments; i++) {
 	    if ((twiceBW < rectPtr->width) && (twiceBW < rectPtr->height)) {
 #if (TK_MAJOR_VERSION == 3)
 		Tk_Draw3DRectangle(graphPtr->display, graphPtr->pixwin,
@@ -740,18 +803,115 @@ DrawNormalBar(graphPtr, elemPtr)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------
+ *
+ * FindActiveIndex --
+ *
+ * 	Performs a binary search for the given index in the array
+ * 	of visible bar segments.  This array contains the indices 
+ * 	of the data points representing a visible (non-clipped)
+ *	segment of the bar element.
+ *
+ * Results:
+ *	Returns the index of the entry in the index array which 
+ *	matches *key*.  If the index key is not found, -1 is returned.
+ *
+ * ----------------------------------------------------------------------
+ */
+static int
+FindActiveIndex(barPtr, key)
+    Bar *barPtr;
+    int key;
+{
+    int median;
+    int low, high;		/* Range of indices to search */
+
+    high = barPtr->numVisibleSegments - 1;
+    low = 0;
+    while (low <= high) {
+	median = (low + high) >> 1;
+	if (key < barPtr->indexArr[median]) {
+	    high = median - 1;
+	} else if (key > barPtr->indexArr[median]) {
+	    low = median + 1;
+	} else {
+	    return (median);
+	}
+    }
+    return -1;
+}
+
+
+/*
+ * ----------------------------------------------------------------------
+ *
+ * GetVisibleActiveSegments --
+ *
+ * 	Generates an array of rectangles representing the active, 
+ *	visible segments of a bar element.  Each data point marked
+ *	as active is examined to see if it's currently visible
+ *	on the graph.  We simply need to test if the index is in 
+ *	*indexArr* which contains the indices of the data points 
+ *	representing visible bar segments.  
+ *
+ * Results:
+ *	Returns the number of visible active segments and stores
+ *	in *segPtrPtr*, the malloc-ed array of rectangles representing
+ *	those segments.  The caller will be responsible for freeing 
+ *	the memory.
+ *
+ * ----------------------------------------------------------------------
+ */
+static int
+GetVisibleActiveSegments(barPtr, segPtrPtr)
+    Bar *barPtr;
+    XRectangle **segPtrPtr;
+{
+    register int i;
+    register XRectangle *rectPtr;
+    XRectangle *rectArr;
+    int numActive;
+    int activeIndex, segIndex;
+    int numDataPoints;
+    
+    /* Allocate array of active bars */
+    rectArr = (XRectangle *)malloc(sizeof(XRectangle) * 
+	barPtr->numActiveIndices);
+    if (rectArr == NULL) {
+	Panic("can't allocate array of active points");
+    }
+    numActive = 0;
+    rectPtr = rectArr;
+    numDataPoints = ELEMENT_LENGTH(barPtr);
+    for (i = 0; i < barPtr->numActiveIndices; i++) {
+	activeIndex = barPtr->activeIndexArr[i];
+	if ((activeIndex < 0) || (activeIndex >= numDataPoints)) {
+	    continue;	/* No such index */
+	}
+	segIndex = FindActiveIndex(barPtr, activeIndex);
+	if (segIndex >= 0) {
+	    *rectPtr = barPtr->segArr[segIndex];
+	    numActive++;
+	    rectPtr++;
+	}
+    } 
+    *segPtrPtr = rectArr;
+    if (numActive == 0) {
+	free((char *)rectArr);
+    }
+    return (numActive);
+}
+
 /*
  * ----------------------------------------------------------------------
  *
  * DrawActiveBar --
  *
- *	Draws the rectangle representing the bar element.  If the
- *	-relief option is set to "raised" or "sunken" and the bar
- *	borderwidth is set (borderwidth > 0), a 3D border is drawn
- *	around the bar.
- *
- *	Don't draw bars that aren't visible (i.e. within the limits
- *	of the axis).
+ *	Draws rectangles representing the active segments of the
+ *	bar element.  If the -relief option is set (other than "flat")
+ *	and the borderwidth is greater than 0, a 3D border is drawn
+ *	around the each bar segment.
  *
  * Results:
  *	None.
@@ -767,40 +927,53 @@ DrawActiveBar(graphPtr, elemPtr)
     Element *elemPtr;
 {
     Bar *barPtr = (Bar *)elemPtr;
-    int twiceBW;
-    register XRectangle *rectPtr;
-    register int i;
+    XRectangle *activePtr;
+    int numActive;
 
-    twiceBW = (2 * barPtr->borderWidth);
-    rectPtr = barPtr->segArr;
-    for (i = 0; i < barPtr->numDataPoints; i++) {
-	if (IsClipped(barPtr->statusArr[i])) {
-	    continue;		/* Data point isn't visible */
-	}
-	if (IsActive(barPtr->statusArr[i])) {
-	    XFillRectangle(graphPtr->display, graphPtr->pixwin,
-		barPtr->activeGC, rectPtr->x, rectPtr->y,
-		rectPtr->width, rectPtr->height);
+    if (barPtr->numActiveIndices > 0) {
+	numActive = GetVisibleActiveSegments(barPtr, &activePtr);
+    } else {
+	/* 
+	 * If numActiveIndices is -1, then the entire element has been
+	 * marked active.  
+	 */
+	activePtr = barPtr->segArr;
+	numActive = barPtr->numVisibleSegments;
+    } 
+    if (numActive == 0) {
+	return;
+    }
+
+    XFillRectangles(graphPtr->display, graphPtr->pixwin, barPtr->activeGC,
+	activePtr, numActive);
+
+    if ((barPtr->borderWidth > 0) && (barPtr->relief != TK_RELIEF_FLAT)) {
+	int twiceBW;
+	register int i;
+
+	twiceBW = (2 * barPtr->borderWidth);
+	for (i = 0; i < numActive; i++) {
 	    /*
 	     * Don't draw a 3D border if it will cover the surface of the bar
 	     */
-	    if ((barPtr->borderWidth > 0) &&
-		(barPtr->relief != TK_RELIEF_FLAT) &&
-		(twiceBW < rectPtr->width) && (twiceBW < rectPtr->height)) {
+	    if ((twiceBW < activePtr->width) && (twiceBW < activePtr->height)) {
 #if (TK_MAJOR_VERSION == 3)
 		Tk_Draw3DRectangle(graphPtr->display, graphPtr->pixwin,
-		    barPtr->border, rectPtr->x, rectPtr->y,
-		    rectPtr->width, rectPtr->height,
+		    barPtr->border, activePtr->x, activePtr->y,
+		    activePtr->width, activePtr->height,
 		    barPtr->borderWidth, barPtr->relief);
 #else
 		Tk_Draw3DRectangle(graphPtr->tkwin, graphPtr->pixwin,
-		    barPtr->border, rectPtr->x, rectPtr->y,
-		    rectPtr->width, rectPtr->height,
+		    barPtr->border, activePtr->x, activePtr->y,
+		    activePtr->width, activePtr->height,
 		    barPtr->borderWidth, barPtr->relief);
 #endif /* TK_MAJOR_VERSION == 3 */
 	    }
+	    activePtr++;
 	}
-	rectPtr++;
+    }
+    if (activePtr != barPtr->segArr) {
+	free((char *)activePtr);
     }
 }
 
@@ -866,67 +1039,9 @@ PrintSymbols(graphPtr, elemPtr, size, pointArr, numPoints, active)
  *
  * PrintActiveBar --
  *
- *	Similar to the DrawElement procedure, prints PostScript
- *	related commands to form rectangle representing the bar
- *	element.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	PostScript pen width, dashes, and color settings are changed.
- *
- * ----------------------------------------------------------------------
- */
-/*ARGSUSED*/
-static void
-PrintBar(graphPtr, barPtr, fgColorPtr, stipple)
-    Graph *graphPtr;
-    Bar *barPtr;
-    XColor *fgColorPtr;
-    Pixmap stipple;
-{
-    register XRectangle *rectPtr;
-    register int i;
-
-    rectPtr = barPtr->segArr;
-    for (i = 0; i < barPtr->numDataPoints; i++) {
-	if (IsClipped(barPtr->statusArr[i])) {
-	    continue;
-	}
-	if (stipple != None) {
-	    int width, height;
-
-	    Blt_BackgroundToPostScript(graphPtr,
-		Tk_3DBorderColor(barPtr->border));
-	    Blt_RectangleToPostScript(graphPtr, rectPtr->x, rectPtr->y,
-		(int)rectPtr->width, (int)rectPtr->height);
-	    Tk_SizeOfBitmap(graphPtr->display, stipple, &width, &height);
-	    Blt_ForegroundToPostScript(graphPtr, fgColorPtr);
-	    Blt_StippleToPostScript(graphPtr, stipple, width, height, True);
-	} else {
-	    Blt_ForegroundToPostScript(graphPtr, fgColorPtr);
-	    Blt_RectangleToPostScript(graphPtr, rectPtr->x, rectPtr->y,
-		(int)rectPtr->width, (int)rectPtr->height);
-	}
-	if ((barPtr->borderWidth > 0) &&
-	    (barPtr->relief != TK_RELIEF_FLAT)) {
-	    Blt_Print3DRectangle(graphPtr, barPtr->border, rectPtr->x,
-		rectPtr->y, (int)rectPtr->width, (int)rectPtr->height,
-		barPtr->borderWidth, barPtr->relief);
-	}
-	rectPtr++;
-    }
-}
-
-/*
- * ----------------------------------------------------------------------
- *
- * PrintActiveBar --
- *
- *	Similar to the DrawElement procedure, prints PostScript
- *	related commands to form rectangle representing the bar
- *	element.
+ *	Similar to the PrintNormalBar procedure, generates PostScript
+ *	commands to display the rectangles representing the active bar
+ *	segments of the element.
  *
  * Results:
  *	None.
@@ -943,8 +1058,51 @@ PrintActiveBar(graphPtr, elemPtr)
     Element *elemPtr;
 {
     Bar *barPtr = (Bar *)elemPtr;
+    XRectangle *activePtr;
+    register int i;
+    int numActive;
 
-    PrintBar(graphPtr, barPtr, barPtr->activeFg, barPtr->activePixmap);
+    if (barPtr->numActiveIndices > 0) {
+	numActive = GetVisibleActiveSegments(barPtr, &activePtr);
+    } else {
+	/* 
+	 * If numActiveIndices is -1, then the entire element has been
+	 * marked active.  
+	 */
+	activePtr = barPtr->segArr;
+	numActive = barPtr->numVisibleSegments;
+    } 
+    if (numActive == 0) {
+	return;
+    }
+    for (i = 0; i < numActive; i++) {
+	if (barPtr->activePixmap != None) {
+	    int width, height;
+
+	    Blt_BackgroundToPostScript(graphPtr,
+		Tk_3DBorderColor(barPtr->border));
+	    Blt_RectangleToPostScript(graphPtr, activePtr->x, activePtr->y,
+		(int)activePtr->width, (int)activePtr->height);
+	    Tk_SizeOfBitmap(graphPtr->display, barPtr->activePixmap,
+		&width, &height);
+	    Blt_ForegroundToPostScript(graphPtr, barPtr->activeFg);
+	    Blt_StippleToPostScript(graphPtr, barPtr->activePixmap,
+		width, height, True);
+	} else {
+	    Blt_ForegroundToPostScript(graphPtr, barPtr->activeFg);
+	    Blt_RectangleToPostScript(graphPtr, activePtr->x, activePtr->y,
+		(int)activePtr->width, (int)activePtr->height);
+	}
+	if ((barPtr->borderWidth > 0) && (barPtr->relief != TK_RELIEF_FLAT)) {
+	    Blt_Print3DRectangle(graphPtr, barPtr->border, activePtr->x,
+		activePtr->y, (int)activePtr->width, (int)activePtr->height,
+		barPtr->borderWidth, barPtr->relief);
+	}
+	activePtr++;
+    }
+    if (activePtr != barPtr->segArr) {
+	free((char *)activePtr);
+    }
 }
 
 /*
@@ -952,9 +1110,8 @@ PrintActiveBar(graphPtr, elemPtr)
  *
  * PrintNormalBar --
  *
- *	Similar to the DrawElement procedure, prints PostScript
- *	related commands to form rectangle representing the bar
- *	element.
+ *	Generates PostScript commands to form the rectangles
+ *	representing the segments of the bar element.
  *
  * Results:
  *	None.
@@ -971,8 +1128,39 @@ PrintNormalBar(graphPtr, elemPtr)
     Element *elemPtr;
 {
     Bar *barPtr = (Bar *)elemPtr;
+    register XRectangle *rectPtr;
+    register int i;
 
-    PrintBar(graphPtr, barPtr, barPtr->normalFg, barPtr->normalPixmap);
+    rectPtr = barPtr->segArr;
+    for (i = 0; i < barPtr->numVisibleSegments; i++) {
+	if ((rectPtr->width < 1) || (rectPtr->height < 1)) {
+	    continue;
+	}
+	if (barPtr->normalPixmap != None) {
+	    int width, height;
+
+	    Blt_BackgroundToPostScript(graphPtr,
+		Tk_3DBorderColor(barPtr->border));
+	    Blt_RectangleToPostScript(graphPtr, rectPtr->x, rectPtr->y,
+		(int)rectPtr->width, (int)rectPtr->height);
+	    Tk_SizeOfBitmap(graphPtr->display, barPtr->normalPixmap,
+		&width, &height);
+	    Blt_ForegroundToPostScript(graphPtr, barPtr->normalFg);
+	    Blt_StippleToPostScript(graphPtr, barPtr->normalPixmap,
+		width, height, True);
+	} else {
+	    Blt_ForegroundToPostScript(graphPtr, barPtr->normalFg);
+	    Blt_RectangleToPostScript(graphPtr, rectPtr->x, rectPtr->y,
+		(int)rectPtr->width, (int)rectPtr->height);
+	}
+	if ((barPtr->borderWidth > 0) &&
+	    (barPtr->relief != TK_RELIEF_FLAT)) {
+	    Blt_Print3DRectangle(graphPtr, barPtr->border, rectPtr->x,
+		rectPtr->y, (int)rectPtr->width, (int)rectPtr->height,
+		barPtr->borderWidth, barPtr->relief);
+	}
+	rectPtr++;
+    }
 }
 
 /*
@@ -1005,21 +1193,24 @@ DestroyBar(graphPtr, elemPtr)
     if (barPtr->activeGC != NULL) {
 	Tk_FreeGC(graphPtr->display, barPtr->activeGC);
     }
-    if (barPtr->segArr != barPtr->defSpace) {
+    if (barPtr->segArr != barPtr->defSegSpace) {
 	free((char *)barPtr->segArr);
     }
     if (barPtr->x.clientId != NULL) {
 	Blt_FreeVectorId(barPtr->x.clientId);
-    } else if (barPtr->x.data != NULL) {
-	free((char *)barPtr->x.data);
+    } else if (barPtr->x.vector.valueArr != NULL) {
+	free((char *)barPtr->x.vector.valueArr);
     }
     if (barPtr->y.clientId != NULL) {
 	Blt_FreeVectorId(barPtr->y.clientId);
-    } else if (barPtr->y.data != NULL) {
-	free((char *)barPtr->y.data);
+    } else if (barPtr->y.vector.valueArr != NULL) {
+	free((char *)barPtr->y.vector.valueArr);
     }
-    if (barPtr->statusArr != NULL) {
-	free((char *)barPtr->statusArr);
+    if (barPtr->activeIndexArr != NULL) {
+	free((char *)barPtr->activeIndexArr);
+    }
+    if (barPtr->indexArr != barPtr->defIndexSpace) {
+	free((char *)barPtr->indexArr);
     }
     free((char *)barPtr);
 }
@@ -1046,7 +1237,7 @@ Blt_BarElement()
 
     barPtr = (Bar *)calloc(1, sizeof(Bar));
     if (barPtr == NULL) {
-	panic("can't allocate a bar element structure");
+	Panic("can't allocate a bar element structure");
     }
     barPtr->configSpecs = configSpecs;
     barPtr->configProc = ConfigureBar;
@@ -1063,8 +1254,8 @@ Blt_BarElement()
     barPtr->type = ELEM_BAR;
     barPtr->relief = TK_RELIEF_RAISED;
     barPtr->borderWidth = 2;
-    barPtr->numDataPoints = barPtr->numSegs = 0;
-    barPtr->segArr = barPtr->defSpace;
+    barPtr->segArr = barPtr->defSegSpace;
+    barPtr->indexArr = barPtr->defIndexSpace;
     return ((Element *)barPtr);
 }
 
@@ -1096,13 +1287,13 @@ Blt_InitFreqTable(graphPtr)
     Tcl_HashEntry *hPtr;
     Tcl_HashSearch cursor;
     Bar *barPtr;
-    register int i;
-    int isNew;
-    int count;
+    int isNew, count;
     int numStacks, numSegs;
     int numDataPoints;
-    FreqKey key, *keyPtr;
-
+    FreqKey key;
+    Tcl_HashTable freqTable;
+    register int i;
+    double *xArr;
     /*
      * Free resources associated with a previous frequency table. This
      * includes the array of frequency information and the table itself
@@ -1119,12 +1310,14 @@ Blt_InitFreqTable(graphPtr)
 	return;			/* No frequency table is needed for
 				 * normal mode */
     }
+    Tcl_InitHashTable(&(graphPtr->freqTable), sizeof(FreqKey) / sizeof(int));
+
     /*
-     * Re-initialize the table and fill it with unique abscissas.
+     * Initialize a hash table and fill it with unique abscissas.
      * Keep track of the frequency of each x-coordinate and how many
      * abscissas have duplicate mappings.
      */
-    Tcl_InitHashTable(&(graphPtr->freqTable), sizeof(FreqKey) / sizeof(int));
+    Tcl_InitHashTable(&freqTable, sizeof(FreqKey) / sizeof(int));
     numSegs = numStacks = 0;
     key.dummy = 0;
     for (itemPtr = Blt_FirstListItem(&(graphPtr->elemList));
@@ -1135,14 +1328,14 @@ Blt_InitFreqTable(graphPtr)
 	}
 	numSegs++;
 	barPtr = (Bar *)elemPtr;
-	numDataPoints = BLT_MIN(barPtr->x.length, barPtr->y.length);
+	xArr = barPtr->x.vector.valueArr;
+	numDataPoints = ELEMENT_LENGTH(barPtr);
 	for (i = 0; i < numDataPoints; i++) {
-	    key.value = barPtr->x.data[i];
+	    key.value = xArr[i];
 	    key.mask = barPtr->axisMask;
-	    hPtr = Tcl_CreateHashEntry(&(graphPtr->freqTable), (char *)&key,
-		&isNew);
+	    hPtr = Tcl_CreateHashEntry(&freqTable, (char *)&key, &isNew);
 	    if (hPtr == NULL) {
-		panic("can't allocate freqTable entry");
+		Panic("can't allocate freqTable entry");
 	    }
 	    if (isNew) {
 		count = 1;
@@ -1160,43 +1353,31 @@ Blt_InitFreqTable(graphPtr)
 	return;			/* No bar elements to be displayed */
     }
     if (numStacks > 0) {
-	FreqInfo *freqArr, *infoPtr;
+	FreqInfo *infoPtr;
+	FreqKey *keyPtr;
+	Tcl_HashEntry *h2Ptr;
 
-	/*
-	 * Prune out singleton entries.  The only consideration here is
-	 * memory consumption.
-	 */
-	for (hPtr = Tcl_FirstHashEntry(&(graphPtr->freqTable), &cursor);
-	    hPtr != NULL; hPtr = Tcl_NextHashEntry(&cursor)) {
+	graphPtr->freqArr = (FreqInfo *)calloc(numStacks, sizeof(FreqInfo));
+	if (graphPtr->freqArr == NULL) {
+	    Panic("can't allocate dup info array");
+	}
+	infoPtr = graphPtr->freqArr;
+	for (hPtr = Tcl_FirstHashEntry(&freqTable, &cursor); hPtr != NULL;
+	    hPtr = Tcl_NextHashEntry(&cursor)) {
 	    count = (int)Tcl_GetHashValue(hPtr);
-	    keyPtr = (FreqKey *)Tcl_GetHashKey(&(graphPtr->freqTable), hPtr);
-	    if (count == 1) {
-		Tcl_DeleteHashEntry(hPtr);
+	    keyPtr = (FreqKey *)Tcl_GetHashKey(&freqTable, hPtr);
+	    if (count > 1) {
+		h2Ptr = Tcl_CreateHashEntry(&(graphPtr->freqTable),
+		    (char *)keyPtr, &isNew);
+		count = (int)Tcl_GetHashValue(hPtr);
+		infoPtr->freq = count;
+		infoPtr->mask = keyPtr->mask;
+		Tcl_SetHashValue(h2Ptr, (ClientData)infoPtr);
+		infoPtr++;
 	    }
 	}
-	freqArr = (FreqInfo *)calloc(numStacks, sizeof(FreqInfo));
-	if (freqArr == NULL) {
-	    panic("can't allocate dup info array");
-	}
-	/*
-	 * Convert the data of each hash entry into an information
-	 * structure.  Save the frequency count.
-	 */
-	infoPtr = freqArr;
-	for (hPtr = Tcl_FirstHashEntry(&(graphPtr->freqTable), &cursor);
-	    hPtr != NULL; hPtr = Tcl_NextHashEntry(&cursor)) {
-	    count = (int)Tcl_GetHashValue(hPtr);
-	    infoPtr->freq = count;
-	    keyPtr = (FreqKey *)Tcl_GetHashKey(&(graphPtr->freqTable), hPtr);
-	    infoPtr->mask = keyPtr->mask;
-	    Tcl_SetHashValue(hPtr, (ClientData)infoPtr);
-	    infoPtr++;
-	}
-	graphPtr->freqArr = freqArr;
-
-    } else {
-	Tcl_DeleteHashTable(&(graphPtr->freqTable));
     }
+    Tcl_DeleteHashTable(&freqTable);
     graphPtr->numStacks = numStacks;
 }
 
@@ -1234,6 +1415,7 @@ Blt_ComputeStacks(graphPtr)
     int numDataPoints;
     register int i;
     register FreqInfo *infoPtr;
+    double *xArr, *yArr;
 
     if ((graphPtr->mode != MODE_STACKED) || (graphPtr->numStacks == 0)) {
 	return;
@@ -1256,16 +1438,18 @@ Blt_ComputeStacks(graphPtr)
 	    continue;
 	}
 	barPtr = (Bar *)elemPtr;
-	numDataPoints = BLT_MIN(barPtr->x.length, barPtr->y.length);
+	xArr = barPtr->x.vector.valueArr;
+	yArr = barPtr->y.vector.valueArr;
+	numDataPoints = ELEMENT_LENGTH(barPtr);
 	for (i = 0; i < numDataPoints; i++) {
-	    key.value = barPtr->x.data[i];
+	    key.value = xArr[i];
 	    key.mask = barPtr->axisMask;
 	    hPtr = Tcl_FindHashEntry(&(graphPtr->freqTable), (char *)&key);
 	    if (hPtr == NULL) {
 		continue;
 	    }
 	    infoPtr = (FreqInfo *)Tcl_GetHashValue(hPtr);
-	    infoPtr->sum += barPtr->y.data[i];
+	    infoPtr->sum += yArr[i];
 	}
     }
 }
